# ***** BEGIN LICENSE BLOCK *****
# * Copyright (C) 2007  Alexey Gladkov <legion@altlinux.org>
# *
# * This program is free software; you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation; either version 2 of the License, or
# * (at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program; if not, write to the Free Software
# * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
# ***** END LICENSE BLOCK *****

. shell-error

> "$workdir/prog.sed"
> "$workdir/block"

format_section() {
	local f="$1" line= first=1 format=
	while read -r line; do
		[ -n "$first" ] && first= && format='%s\n' || format='\t%s\n'
		printf "$format" "$line"
	done < "$f"
}

show_global_param() {
	param="$1"
	sed -rne "/^$param\$/{p;q}" -e "s/^$param=\"(.*)\"\$/\1/p" "$globals"
}

show_section() {
	local liloconf= label= curtype= line= section= lbl= found=

	[ "$#" -gt 1 ] ||
		fatal "lilo config file required"
	liloconf="$1"; shift

	[ "$#" -eq 1 ] ||
		fatal "label required"
	label="$1"; shift

	while read -r line; do
		[ -n "$line" ] || continue
		case "${line%%=*}" in
			image|other) [ -z "$found" ] && section= || break ;;
			label)
				lbl="${line#label=}"
				lbl="${lbl#\"}"
				lbl="${lbl%\"}"
				[ "$lbl" != "$label" ] || found=1
				;;
		esac
		section="${section:+$section
}$line"
	done < "$liloconf"
	[ -z "$found" ] || printf %s\\n "$section"
}

show_section_type() {
	grep -qs '^other=' "$1" &&
		printf other ||
		printf image
}

show_globals() {
	[ "$#" -eq 1 ] ||
		fatal "lilo config file required"

	while read -r line; do
		[ -n "$line" ] || continue
		case "${line%%=*}" in
			image|other) break ;;
		esac
		printf %s\\n "$line"
	done < "$1"
}

show_default() {
	local def_label="$(show_global_param 'default')"
	[ -n "$def_label" ] ||
		def_label="$(head -1 "$order" |cut -f2)"
	printf %s "$def_label"
}

show_sections() {
	[ "$#" -eq 1 ] ||
		fatal "lilo config file required"

	local line= type= label=
	while read -r line; do
		[ -n "$line" ] || continue

		case "${line%%=*}" in
			image|other)
				local f="${line#*=}"
				f="${f%\"*}"
				f="${f#*\"}"
				f="$LILO_DESTDIR/$f"
				[ -b "$f" -o -s "$f" ] &&
				  type="${line%%=*}" ||
				  type=""
				;;
			label)
				label="${line#label=}"
				label="${label#\"}"
				label="${label%\"}"

				[ -z "$type" ] ||
					printf '%s\t%s\n' "$type" "$label"

				label= type=
				;;
		esac
	done < "$1"
}

test_other_section(){
  local dev="$1"
  local cnf="$2"
  #is section with device $dev exists in lilo.conf

  local line= device= label=
  while read -r line; do
    [ -n "$line" ] || continue
    case "${line%%=*}" in
      other)
        device="${line#other=}"
        device="${device#\"}"
        device="${device%\"}"
      ;;
      label)
        label="${line#label=}"
        label="${label#\"}"
        label="${label%\"}"
        [ -z "$device" ] || [ "$device" != "$dev" ] || echo "$label"
        label= device=
      ;;
    esac
  done < "$cnf"
}


parse_globals() {
	[ -n "$1" ] ||
		fatal "lilo config file required"

	show_globals "$1" > "$globals"
	# "$2" eq "no" when lilo.conf is created
	[ "$2" = "no"  ] && cmdline_params
	detect_raidroot
}

parse_sections() {
	local label= sect_type=

	[ "$#" -eq 1 ] ||
		fatal "lilo config file required"

	show_sections "$1" >"$order"
	[ ! -d "$sections" ] || rm -rf -- "$sections"
	mkdir -p -- "$sections"
	while read sect_type label; do
		show_section "$1" "$label" > "$sections/$label"
	done < "$order"

}

parse_liloconf() {
	[ -n "$1" ] ||
		fatal "lilo config file required"
	# "$2" eq "no" when lilo.conf is created
	parse_globals  "$1" "$2"
	parse_sections "$1"
}

write_params() {
	local keyword dummy
	sed -r -e 's#[[:space:]]+# #g' -e 's#^([^=]+=)"(.*)"$#\1 \2#' |
	while read -r keyword dummy; do
		[ -n "$keyword" ] || continue

		if [ -z "${keyword##*=}" ]; then
#			[ -n "$dummy" ] || continue
			dummy="\"$(quote_shell "$dummy")\""
		fi
		printf '%s%s\n' "$keyword" "$dummy"
	done
}

remove_global() {
	local k="$(quote_sed_regexp "$1")"
	printf '/^%s/d\n' "$1" >> "$workdir/prog.sed"
}

replace_global() {
	remove_global "$1"
	[ -z "$2" ] || printf %s\\n "$2" >> "$workdir/block"
}

commit() {
	if [ -s "$workdir/prog.sed" ]; then
		sed -i -f "$workdir/prog.sed" "$1"
		> "$workdir/prog.sed"
	fi
	grep -v '^[[:space:]]*$' "$workdir/block" |write_params >> "$1"
	> "$workdir/block"
}

reset_changes() {
	[ "$#" -eq 2 ] ||
		fatal "Usage: reset_changes <config> <type>"

	case "$2" in
		sections)
			local def="$(show_default)"
			replace_global "default" "default=\"$def\""
			parse_sections "$1"
			;;
		globals) parse_globals "$1" "yes"
			;;
	esac
	commit "$globals"
}

write_globals() {
	[ "$#" -eq 1 ] ||
		fatal "Usage: write_globals <target>"

	case "$1" in
		boot)
			[ "$in_devname" != '#t' ] || in_devname="$(get_default_boot_dev)"
			if [ -n "$raidboot" ]; then
				[ "$raidboot" = "$in_devname" ] &&
					replace_global "raid-extra-boot" "raid-extra-boot=\"mbr-only\"" ||
					remove_global "raid-extra-boot"
			fi
			replace_global "boot" "boot=\"$in_devname\""
			;;
		default)
			replace_global "default" "default=\"$in_default\""
			;;
		globals)
			if [ "$in_method" != "insecure" ]; then
				replace_global "password" "password=\"$in_password1\""
				replace_global "\(restricted\|mandatory\)" "$in_method"
			else
				remove_global "\(restricted\|mandatory\|password\)"
			fi
			in_extraboot="$(printf %s "$in_extraboot" |tr -d '[:space:]')"
			[ -n "$in_extraboot" ] &&
				replace_global "raid-extra-boot" "raid-extra-boot=\"$in_extraboot\"" ||
				remove_global "raid-extra-boot"

			replace_global "boot" "boot=\"$in_boot\""
			replace_global "timeout" "timeout=\"$in_timeout\""
			replace_global "install" "install=\"$in_install\""
			if [ "$in_install" = "text" ]; then
			  remove_global "message"
			else
			  replace_global "message" "message=\"/boot/splash/message\""
			fi
			replace_global "append" "append=\"$in_append\""
			;;
		timeout)
			replace_global "timeout" "timeout=\"$in_timeout\""
			;;
	esac
	commit "$globals"
}

remove_fields_from_section() {
  if [ -z "$1" -o ! -f "$sections/$1" ]; then
    write_error "`_ "Unknown section"` $1."
    return 1
  fi
  local type=$(show_section_type "$sections/$1")
  case "$type" in
    image) 
      awk '!/(^image=)|(^label=)|(^root=)|(^initrd=)|(^addappend=)|(^alias=)/{print}'\
        < "$sections/$1" > "$tmp_section"
      mv "$tmp_section" "$sections/$1"
      ;;
    other)
      awk '!/(^other=)|(^label=)|(^alias=)|(^table=)|(^master-boot)/{print}' \
        < "$sections/$1" > "$tmp_section"
      mv "$tmp_section" "$sections/$1"
      ;;
    esac
}

add_fields_to_image_section() {
	{
		printf 'image="%s"\n' "$in_image"
		printf 'label="%s"\n' "$in_label"
		printf 'root="%s"\n' "$in_root"
		[ -z "$in_initrd" ] || printf 'initrd="%s"\n' "$in_initrd"
		[ -z "$in_addappend" ] || printf 'addappend="%s"\n' "$in_addappend"
		[ -z "$in_alias" ]  || printf 'alias="%s"\n' "$in_alias"
	} | write_params
}

add_fields_to_other_section() {
	{
		printf 'other="%s"\n' "$in_other"
		printf 'label="%s"\n' "$in_label"
		[ -z "$in_alias" ]   || printf 'alias="%s"\n' "$in_alias"
		[ -z "$in_table" ]   || printf 'table="%s"\n' "$in_table"
		[ "$in_master_boot" != "#t" ] || printf 'master-boot\n'
	} | write_params
}




not_section() {
	if [ -f "$sections/$1" ]; then
		write_error "`_ "Section already exists."`"
		return 1
	fi
}

write_section() {
  [ "$#" -eq 2 ] ||
    fatal "Usage: write_section <oldlabel> <newlabel>"


  local newtype="${in_type:-image}"
  local old="$(quote_sed_regexp "$1")"
  local new="$(quote_sed_regexp "$2")"

  if [ -z "$new" ]; then
    write_error "`_ "Label is empty."`"
    return
  fi

  if [ "$newtype" = "other" -a -z "${in_other:-}" ]; then
    write_error "`_ "Other loader device is unspecified."`"
    return
  fi

  if [ "$newtype" = "image" -a -z "${in_image:-}" ]; then
    write_error "`_ "Kernel image is unspecified."`"
    return
  fi

  if [ "$newtype" = "image" -a -z "${in_root:-}" ]; then
    write_error "`_ "Root device is unspecified."`"
    return
  fi

  # editing old section:
  if [ -n "$1" ]; then
    # changing section name:
    if [ "$1" != "$2" ]; then
      not_section "$2" || return 1 # target section exists
      mv -f -- "$sections/$1" "$sections/$2"
    fi
    remove_fields_from_section "$2"
    sed -i -e "s#^\(.*[[:space:]]\)$old\$#\1$new#g" "$order"
    sed -i -e "s#^\(default=\)\"$old\"\$#\1\"$new\"#" "$globals"
  else # creating new section
    not_section "$2" || return 1 # target section exists
    printf '%s\t%s\n' "$newtype" "$new" >> "$order"
  fi
  case "$newtype" in
    image)  add_fields_to_image_section > "$sections/$2" ;;
    other)  add_fields_to_other_section > "$sections/$2" ;;
  esac
  return 0
}

remove_section() {
	[ "$#" -eq 1 ] ||
		fatal "Usage: remove_section <label>"

	[ -n "$1" ] || return 0
	local old="$(quote_sed_regexp "$1")"
	sed -i -e "\#^.*[[:space:]]$old\$#d" "$order"
	rm -f -- "$sections/$1"
}

show_params_list() {
	local keyword dummy
	sed -r -e 's#[[:space:]]+# #g' -e 's#^[[:space:]]*([^=]+=)"(.*)"$#\1 \2#' "$1" |
	while read -r keyword dummy; do
		[ -n "$keyword" -a -n "${keyword%%\#*}" ] || continue
		keyword="${keyword%=}"
		keyword="${keyword//-/_}"
		if [ -n "$dummy" ]; then
		  write_string_param "$keyword" "$dummy"
		else
		  write_bool_param "$keyword" "yes"
		fi
	done
}

get_curr_dev(){
  cur_dev="$(sed -rne 's,^boot=\"(.*)\"$,\1,p' "$globals")" || return 0
  echo "/dev/${cur_dev##/dev/}"
}
