#!/bin/sh -efu

. ../alterator-lc-functions

lc_test(){
  local i=
  if [ "${1:-}" = "-i" ]; then
     i="$1"; shift
  fi
  local func="$1"
  local inp="$2"
  local exp="$3"
  shift 3

  local sep="\n=====\n"
  local F="$(mktemp)"

  printf "$inp" > "$F"

  local out ret=0
  out="$(eval '$func $i "$F" "$@"')" || ret="$?"

  [ -z "$i" ] || out="$(cat "$F")"

  if [ "$out" != "$exp" ]; then
    printf "Error: $func $i $@$sep%s$sep%s$sep" "$out" "$exp"
    exit 1
  fi
  return "$ret"
}

### splitting lilo.conf into sections

i='#global section
  some text
image=section1
  section1
 other = section2
section 2 data'

o='<<<#global section
  some text>>>

<<<image=section1
  section1>>>

<<< other = section2
section 2 data>>>'

lc_test liloconf_select_sections "$i" "$o"

### last section contains only one line, empty global section

i='image=section 1
  section1
#image=section 1a
  section1a
other = section 2'

o='<<<>>>

<<<image=section 1
  section1
#image=section 1a
  section1a>>>

<<<other = section 2>>>'

lc_test liloconf_select_sections "$i" "$o"

### get one section

i='#global section
  some text

image=section1
  section1

 other = section2

 other = /some/path/section3

image = /some/path/section4#comment

 other = section5
   label = s5 # test
    data

other = section6a
   data
   label = s6 # test
other = section6b
   label = s6 # test
'

o='#global section
  some text'
lc_test liloconf_get_section "$i" "$o" ""

o='image=section1
  section1'
lc_test liloconf_get_section "$i" "$o" "section1"

o=' other = section2'
lc_test liloconf_get_section "$i" "$o" "section2"

o=' other = /some/path/section3'
lc_test liloconf_get_section "$i" "$o" "section3"

o='image = /some/path/section4#comment'
lc_test liloconf_get_section "$i" "$o" "section4"

o=' other = section5
   label = s5 # test
    data'
lc_test liloconf_get_section "$i" "$o" "s5"

# broken lilo.conf: sections with the same name. Get only first one
o='other = section6a
   data
   label = s6 # test'
lc_test liloconf_get_section "$i" "$o" "s6"

### list sections
o='section1
section2
section3
section4
s5
s6
s6'
lc_test liloconf_list_sections "$i" "$o"

### delete section

o='image=section1
  section1

 other = section2

 other = /some/path/section3

image = /some/path/section4#comment

 other = section5
   label = s5 # test
    data

other = section6a
   data
   label = s6 # test
other = section6b
   label = s6 # test'
lc_test liloconf_del_section "$i" "$o" ""
lc_test -i liloconf_del_section "$i" "$o" ""

o='#global section
  some text

 other = section2

 other = /some/path/section3

image = /some/path/section4#comment

 other = section5
   label = s5 # test
    data

other = section6a
   data
   label = s6 # test
other = section6b
   label = s6 # test'
lc_test liloconf_del_section "$i" "$o" "section1"
lc_test -i liloconf_del_section "$i" "$o" "section1"

o='#global section
  some text

image=section1
  section1

 other = section2

 other = /some/path/section3

image = /some/path/section4#comment

 other = section5
   label = s5 # test
    data'
lc_test liloconf_del_section "$i" "$o" "s6"
lc_test -i liloconf_del_section "$i" "$o" "s6"


### parameters
i='  #comment1
vga="0x314"
 map="/boot/map"
lba32
#comment2
prompt # comment
compact
install=text#comment
append="panic=30 splash=silent"
boot="/dev/disk/by-id/scsi-SATA_ST3250820AS_5QE2FCJP"
timeout="100"
default="ALTLinux"

image="/boot/vmlinuz"
        label="ALTLinux"
        root="UUID=6ad2c76e-9ca0-410e-a570-45e28eb4020e"
        initrd="/boot/initrd.img"

image="/boot/vmlinuz"
        label="failsafe"
        root="UUID=6ad2c76e-9ca0-410e-a570-45e28eb4020e"
        initrd="/boot/initrd.img"
        addappend="failsafe"

other="/dev/disk/by-id/scsi-SATA_ST3250820AS_5QE2FCJP-part6"
        label="test"

image="/boot/vmlinuz-2.6.25-std-def-alt7" #comm
	initrd="/boot/initrd-2.6.25-std-def-alt7.img"
	label="2625-std-def-7"
	root="UUID=6ad2c76e-9ca0-410e-a570-45e28eb4020e"
	read-only
	read-only
	optional'

lc_test liloconf_get_par "$i" "#t"    "" "compact"
lc_test liloconf_get_par "$i" "#t"    "" "prompt"
lc_test liloconf_get_par "$i" "#f"    "" "missing_par"
lc_test liloconf_get_par "$i" "0x314" "" "vga"
lc_test liloconf_get_par "$i" "text"  "" "install"
lc_test liloconf_get_par "$i" "/dev/disk/by-id/scsi-SATA_ST3250820AS_5QE2FCJP"  "" "boot"
lc_test liloconf_get_par "$i" "panic=30 splash=silent" "" "append"

lc_test liloconf_get_par  "$i" "#t" "2625-std-def-7" "read-only"
lc_test liloconf_get_par  "$i" "#f" "2625-std-def-7" "compact"
lc_test liloconf_get_par  "$i" "#t" "2625-std-def-7" "optional"
lc_test liloconf_get_par  "$i" "UUID=6ad2c76e-9ca0-410e-a570-45e28eb4020e" "2625-std-def-7" "root"
lc_test liloconf_get_par  "$i" "/boot/vmlinuz-2.6.25-std-def-alt7" "2625-std-def-7" "image"

lc_test liloconf_test_par "$i" "" "" "compact" || echo "FAIL: compact"
lc_test liloconf_test_par "$i" "" "" "prompt"  || echo "FAIL: prompt"
lc_test liloconf_test_par "$i" "" "" "missing_par" && echo "FAIL: missing_par" ||:
lc_test liloconf_test_par "$i" "" "" "vga"     || echo "FAIL: vga"
lc_test liloconf_test_par "$i" "" "" "install" || echo "FAIL: install"
lc_test liloconf_test_par "$i" "" "" "append"  || echo "FAIL: append"

lc_test liloconf_test_par  "$i" "" "2625-std-def-7" "read-only" || echo "FAIL: read-only"
lc_test liloconf_test_par  "$i" "" "2625-std-def-7" "compact"   && echo "FAIL: compact" ||:
lc_test liloconf_test_par  "$i" "" "2625-std-def-7" "optional"  || echo "FAIL: optional"
lc_test liloconf_test_par  "$i" "" "2625-std-def-7" "root"      || echo "FAIL: root"
lc_test liloconf_test_par  "$i" "" "2625-std-def-7" "image"     || echo "FAIL: image"

lc_test liloconf_test_par  "$i" "" "2625-std-def" "image" && echo "FAIL: image" ||:
lc_test liloconf_test_par  "$i" "" "nnn" "image" && echo "FAIL: image" ||:

### par_del

i='vga="0x314"
 map="/boot/map"
lba32
#comment2
prompt # comment
compact
install=text#comment
append="panic=30 splash=silent"
boot="/dev/disk/by-id/scsi-SATA_ST3250820AS_5QE2FCJP"
timeout="100"
  default="ALTLinux"

image="/boot/vmlinuz"
        label="ALTLinux"
        root="UUID=6ad2c76e-9ca0-410e-a570-45e28eb4020e"
        initrd="/boot/initrd.img"
'

o="$(printf "$i" | sed '/^vga="0x314"/d')"
lc_test liloconf_del_par "$i" "$o" "" "vga"
lc_test -i liloconf_del_par "$i" "$o" "" "vga"

o="$(printf "$i" | sed '/^compact/d')"
lc_test liloconf_del_par "$i" "$o" "" "compact"
lc_test -i liloconf_del_par "$i" "$o" "" "compact"

o="$(printf "$i" | sed '/^  default="ALTLinux"/d')"
lc_test liloconf_del_par "$i" "$o" "" "default"
lc_test -i liloconf_del_par "$i" "$o" "" "default"

o="$(printf "$i")"
lc_test liloconf_del_par "$i" "$o" "" "default="
lc_test -i liloconf_del_par "$i" "$o" "" "default="

o="$(printf "$i" | sed '/^        root="UUID/d')"
lc_test liloconf_del_par "$i" "$o" "ALTLinux" "root"
lc_test -i liloconf_del_par "$i" "$o" "ALTLinux" "root"

### be careful: you can remove image/other parameter
o="$(printf "$i" | sed '/^image="/d')"
lc_test liloconf_del_par "$i" "$o" "ALTLinux" "image"
lc_test -i liloconf_del_par "$i" "$o" "ALTLinux" "image"

o="$(printf "$i" | sed '/^        initrd="/d')"
lc_test liloconf_del_par "$i" "$o" "ALTLinux" "initrd"
lc_test -i liloconf_del_par "$i" "$o" "ALTLinux" "initrd"

### par_set

i='p1=v1
 p2=v2
 p3 = v3
 p4 = "v4" # comment
p5 = v5 # comment
 p6
p7 # comment
# comment
other="s1oth"
  label="s1"
  p1=v1-1 #

other = "s2"
  p1=v1-2
  p2
#comment
image=s3 #comment
  p1=v1-3
image=/base/s4i
  p1=v1-4
  label="s4"
other=base/s5
'

o="$(printf "$i" | sed 's/^p1=v1$/p1/')"
lc_test liloconf_set_par "$i" "$o" "" "p1"
lc_test -i liloconf_set_par "$i" "$o" "" "p1"

o="$(printf "$i" | sed 's/^ p2=v2$/ p2=""/')"
lc_test liloconf_set_par "$i" "$o" "" "p2" ""
lc_test -i liloconf_set_par "$i" "$o" "" "p2" ""

o="$(printf "$i" | sed 's/^ p3 = v3$/ p3="abc"/')"
lc_test liloconf_set_par "$i" "$o" "" "p3" "abc"
lc_test -i liloconf_set_par "$i" "$o" "" "p3" "abc"

o="$(printf "$i" | sed 's/^ p6$/ p6="abc"/')"
lc_test liloconf_set_par "$i" "$o" "" "p6" "abc"
lc_test -i liloconf_set_par "$i" "$o" "" "p6" "abc"

# comments are removed
o="$(printf "$i" | sed 's/^p7 # comment$/p7="abc"/')"
lc_test liloconf_set_par "$i" "$o" "" "p7" "abc"
lc_test -i liloconf_set_par "$i" "$o" "" "p7" "abc"

o="$(printf "$i" | sed 's/^other=base\/s5$/other="123"/')"
lc_test liloconf_set_par "$i" "$o" "s5" "other" "123"
lc_test -i liloconf_set_par "$i" "$o" "s5" "other" "123"

o="$(printf "$i" | sed 's/^  label="s4"$/  label="X"/')"
lc_test liloconf_set_par "$i" "$o" "s4" "label" "X"
lc_test -i liloconf_set_par "$i" "$o" "s4" "label" "X"
