(document:surround "/std/frame")
(document:surround "/std/functions")

(define (reload-sections)
  (sections enumref "/lilo/sections")
  (default enumref "/lilo/sections"))

(define (read-section)
  (woo-catch/message (thunk
    (let ((data (woo-read-first "/lilo/sections" 'section (sections value))))

      (section-label   value (woo-get-option data 'label ""))

      (section-other   value (woo-get-option data 'other ""))
      (section-alias   value (woo-get-option data 'alias ""))
      (section-mboot   value (woo-get-option data 'master_boot #f))
      (section-table   value (woo-get-option data 'table ""))

      (section-image   value (woo-get-option data 'image ""))
      (section-initrd  value (woo-get-option data 'initrd ""))
      (section-root    value (woo-get-option data 'root ""))
      (section-append  value (woo-get-option data 'addappend ""))

      (section-type value
        (if (string= (section-other value) "") "image" "other"))
      (section-type-label text
        (if (string= (section-other value) "") (_"Linux kernel") (_"Other loader")))
      (update-effect)
))))

(define (add-section type)
    (sections value "")
    (section-type value type)
    (section-type-label text
      (if (string= type "image") (_"Linux kernel") (_"Other loader")))

    (section-label   value "")

    (section-other   value "")
    (section-alias   value "")
    (section-mboot   value #f)
    (section-table   value "")

    (section-image   value "")
    (section-initrd  value "")
    (section-root    value "")
    (section-append  value "")
    (update-effect) )

(define (remove-section)
  (woo-catch/message (thunk
    (woo-write "/lilo/section-remove" 'label (sections value)) ))
  (reload-sections)
  (read-globals)
  (read-section) )

(define (real-write-section)
  (woo-write "/lilo/section"
    'type      (section-type value)
    'oldlabel  (sections value)
    'label     (section-label value)
    'image     (section-image value)
    'initrd    (section-initrd value)
    'root      (section-root value)
    'addappend (section-append value)
    'other     (section-other value)
    'alias     (section-alias value)
    'master_boot (section-mboot value)
    'table       (section-table value)
    ))

;;FIXME: fix catch usage (all calls inside woo-catch/message)
(define (write-section)
  (and (woo-catch/message (thunk (real-write-section)))
       (reload-sections) (sections value (section-label value)) ))

(define (read-globals)
  (woo-catch/message (thunk
    (let ((data (woo-read-first '/lilo/globals)))
      (sections value (woo-get-option data 'default ""))
      (default value (woo-get-option data 'default ""))
      (global-boot value (woo-get-option data 'boot ""))
      (global-timeout value (woo-get-option data 'timeout ""))
      (global-append value (woo-get-option data 'append ""))
      (config-only state (woo-get-option data 'configonly #f))
      (global-install value (woo-get-option data 'install "text"))
      (global-raid-extra-boot value (woo-get-option data 'raid_extra_boot ""))))))

(define (write-globals commit)
  (woo-catch/message (thunk
    (woo-write "/lilo/globals"
       'target   "globals"
       'boot     (global-boot value)
       'append   (global-append value)
       'timeout  (global-timeout value)
       'install  (global-install value)
       'extraboot (global-raid-extra-boot value)
       'method "insecure"
       'confonly (config-only state))
    (woo-write "/lilo/globals" 'target "default" 'default (default value))
    (if commit (woo-write "/lilo/commit" 'confonly (config-only state)))
)))

(define (write-reset)
  (woo-catch/message (thunk
    (woo "reset" "/lilo" 'part "globals")
    (woo "reset" "/lilo" 'part "sections") ))
    (read-globals)
    (reload-sections) )

(define (change url)
  (and (write-globals #f) (frame:replace url)))

;;;
;;; UI

(gridbox columns "0;100;0;0;0" align "top"
  (document:id config-only
    (checkbox (_ "Don't install bootloader") state #f))
  (spacer)
  (button (_ "Simple mode") (when clicked (change "/lilo")))
  (button (_ "Expert mode") activity #f)
  (button (_ "Edit lilo.conf") (when clicked (change "/lilo/edit-source")))
)

(separator)
(label (bold (_ "Common settings")))

(gridbox columns "0;70;0;30" align "top"

  (label (_ "Install boot loader on:") align 'right)
  (document:id global-boot (edit colspan 3))

  (label (_ "Raid-extra-boot:")  align 'right)
  (document:id global-raid-extra-boot (combobox enumref "lilo/raid_extra_boot_list"))

  (spacer colspan 2)

  (label (_ "Interface type:")  align 'right)
  (document:id global-install (combobox enumref "lilo/install_list"))

  (label (_ "Interface timeout:")    align 'right)
  (document:id global-timeout (edit value ""))

  (label (_ "Common kernel parameters:") align 'right)
  (document:id global-append  (edit colspan 3 value ""))

  (spacer colspan 4)

  (label text (_ "Default boot variant:")    align 'right)
  (document:id default (combobox enumref "lilo/sections"))

  (spacer colspan 2)
)

(separator)
(label (bold (_ "Boot variants")))

(gridbox columns "30;0;40" align "bottom"
  (document:id sections (listbox rowspan 2 rowspan 11 (when selected (read-section))))

  (spacer) (document:id section-type-label   (label))
  (label (_ "Label:") align 'right) (document:id section-label   (edit value ""))

  (label text (_ "Kernel image:") name "section-image"  align 'right)
    (document:id section-image (edit value "" name "section-image"))
  (label text (_ "Initrd:") name "section-initrd" align 'right)
    (document:id section-initrd (edit value "" name "section-initrd"))
  (label text (_ "Root device:") name "section-root" align 'right)
    (document:id section-root (edit value "" name "section-root"))
  (label text (_ "Additional parameters:") name "section-append" align 'right)
    (document:id section-append (edit value "" name "section-append"))

  (label text (_ "Device:") name "section-other" align 'right)
    (document:id section-other (edit value "" name "section-other"))
  (label text (_ "Alias:") name "section-alias" align 'right)
    (document:id section-alias (edit value "" name "section-alias"))
  (label text (_ "Table:") name "section-table" align 'right)
    (document:id section-table (edit value "" name "section-table"))
  (spacer)
    (document:id section-mboot (checkbox name "section-mboot" text (_ "Master boot") ))

  (document:id section-type (edit visibility #f name "type"))

  (gridbox columns "0;0;100;0"
    (button (_ "Save")  (when clicked (write-section)))
    (button (_ "Reset") (when clicked (read-section)))
    (spacer)
    (button (_ "Delete") (when clicked (remove-section)))
  )
)

(gridbox columns "0;0;100" align "top"
  (button (_ "Add Linux kernel") (when clicked (add-section "image")))
  (button (_ "Add other loader") (when clicked (add-section "other")))
)

(separator)


;;;;

(or (global 'frame:next)
    (gridbox columns "50;0;0;50" align "top"
        (spacer)
        (button (_ "Apply") (when clicked (write-globals #t)))
        (button (_ "Reset") (when clicked (write-reset)))
        (spacer)))

(and (global 'frame:next)
	(frame:on-next (thunk (or (write-globals #t) 'cancel))))

(document:root (when loaded
  (and (global 'frame:next) (frame:back-activity #f))
  (reload-sections)
  (read-globals)
  (read-section)
  (init-effect)
))

(effect-hide "section-image"    "type" "other")
(effect-hide "section-initrd"   "type" "other")
(effect-hide "section-root"     "type" "other")
(effect-hide "section-append"   "type" "other")

(effect-show "section-other"    "type" "other")
(effect-show "section-alias"    "type" "other")
(effect-show "section-mboot"    "type" "other")
(effect-show "section-table"    "type" "other")

