(document:surround "/std/frame")

;;;
(define (rlist-widgets)
  (map (lambda(data)
         (radio name "devname"
                value (woo-get-option data 'name)
                text (woo-get-option data 'label)))
       (woo-list "/lilo/bootloader_places")))
;;;


;;;
(define cblist (make-cell `()))

(define (cblist-add name label st)
  (cell-set! cblist (cons (cons name (checkbox label state st)) (cell-ref cblist))) )

(define (cblist-get)
  (apply string-append
    (map (lambda (p) (string-append (if ((cdr p) state) "+" "-") (car p) ";"))
         (cell-ref cblist))))

(define (cblist-widgets)
  (reverse (map cdr (cell-ref cblist))))
;;;

(woo-catch/message (thunk
  (map (lambda (data) (cblist-add
          (woo-get-option data 'name "")
          (woo-get-option data 'long_label "")
          (woo-get-option data 'enabled #f) ))
       (woo-list "/lilo/others") )))

;;; read

(define (round-timeout time)
  (if (and (string? time) (not (string= time "")) ); backend can return #t
    (let ((t (string->number time)))
      (cond
        ((< t 10) "0")
        ((< t 50) "10")
        ((< t 100) "50")
        ((< t 200) "100")
        ((< t 600) "200")
        (else "600")))
    ""))

(define (default-section)
  (catch/message
    (lambda()
      (let* ((sects (woo-list "lilo/sections"))
	     (defs  (find (lambda(x)
			    (woo-get-option x 'default #f))
			  sects)))
	(woo-get-option defs 'name)))))

(define (ui-read)
  (form-update-value
    "default" (default-section))
  (form-update-value
    "timeout"
    (round-timeout
      (woo-get-option
	(woo-read-first "lilo/globals")
	'timeout)))
  (form-update-value
    "devname"
    (woo-get-option (woo-read-first "/lilo/dev") 'current)))

;;; write

(define (ui-write)
  (catch/message
    (lambda()
      (woo-write "/lilo/globals" 'target "boot" 'devname (form-value "devname"))
      (woo-write "/lilo/simple_sections" 'sections (cblist-get))
      (woo-write "/lilo/globals" 'target "timeout" 'timeout (form-value "timeout"))
      (woo-write "/lilo/globals" 'target "default" 'default (form-value "default")))))

(define (ui-commit)
  (catch/message
    (lambda()
      (and (ui-write)
	   (woo-write "/lilo/commit" 'confonly #f)))))

(define (ui-expert)
  (ui-write)
  (frame:replace "/lilo/expert"))

;;; UI

(gridbox
  columns "50;0;50"

  (spacer)
  (apply groupbox (_ "Install boot loader on:") (rlist-widgets))
  (spacer)

  (spacer)
  (if (not (= 0 (length (cell-ref cblist))))
    (apply groupbox (_ "Load other OS:") (cblist-widgets))
    (spacer))
  (spacer)

  (spacer)
  (gridbox
    columns "0;100"
    (label colspan 2)

    (label text (_ "Load by default:") align "right")
    (combobox name "default")

    (label text (_ "Interface timeout:") align "right")
    (combobox name "timeout"))
  (spacer)

  (label colspan 3)

  (spacer)
  (gridbox
    columns "50;50"
    align "top"
    (if (global 'frame:next)
      (spacer)
      (button text (_ "Apply") name "apply_button"))
    (button text (_ "Expert mode") name "expert_button"))
  (spacer))

(and (global 'frame:next)
     (frame:on-next (lambda() (or (ui-commit) 'cancel))))

;;; Init

(document:root
  (when loaded
    (form-update-enum "timeout" (woo-list "/lilo/std_timeouts"))
    (form-update-enum "default" (woo-list "/lilo/sections"))
    (ui-read)
    (form-bind "expert_button" "click" ui-expert)
    (or (global 'frame:next) (form-bind "apply_button" "click" ui-commit))
    (and (global 'frame:next) (frame:back-activity #f))))
